package com.younike.service.service;

import com.younike.common.core.dto.Ztree;
import com.younike.service.dto.system.CheckboxDto;
import com.younike.service.pojo.SysMenu;
import com.younike.service.pojo.SysRole;
import com.younike.service.pojo.SysUser;
import com.younike.service.pojo.SysUserMenu;
import com.younike.service.pojo.vo.RouterVo;
import com.younike.service.pojo.vo.TreeSelect;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 菜单 业务层
 */
public interface ISysMenuService {
    /**
     * 根据用户ID查询菜单
     *
     * @param user 用户信息
     * @return 菜单列表
     */
    List<SysMenu> selectMenusByUser(SysUser user);

    /**
     * 根据用户信息查询权限字符串集合
     *
     * @param user 用户信息
     * @return 权限字符串集合
     */
    Set<String> getMenuPermissions(SysUser user);

    /**
     * 查询系统菜单列表
     *
     * @param menu   菜单信息
     * @param userId 用户ID
     * @return 菜单列表
     */
    List<SysMenu> selectMenuList(SysMenu menu, Long userId);

    /**
     * 根据用户查询系统菜单列表
     *
     * @param userId 用户ID
     * @return 菜单列表
     */
    List<SysMenu> selectMenuList(Long userId);

    /**
     * 查询菜单集合
     *
     * @param userId 用户ID
     * @return 所有菜单信息
     */
    List<SysMenu> selectMenuAll(Long userId);

    /**
     * 根据用户ID查询权限
     *
     * @param userId 用户ID
     * @return 权限列表
     */
    Set<String> selectPermsByUserId(Long userId);


    /**
     * 构建前端路由所需要的菜单
     *
     * @param menus 菜单列表
     * @return 路由列表
     */
    public List<RouterVo> buildMenus(List<SysMenu> menus);

    /**
     * 根据角色ID查询菜单
     *
     * @param role   角色对象
     * @param userId 用户ID
     * @return 菜单列表
     */
    List<Ztree> roleMenuTreeData(SysRole role, Long userId);

    /**
     * 查询所有菜单信息
     *
     * @param userId 用户ID
     * @return 菜单列表
     */
    List<Ztree> menuTreeData(Long userId);

    /**
     * 查询系统所有权限
     *
     * @param userId 用户ID
     * @return 权限列表
     */
    Map<String, String> selectPermsAll(Long userId);

    /**
     * 删除菜单管理信息
     *
     * @param menuId 菜单ID
     * @return 结果
     */
    int deleteMenuById(Long menuId);

    /**
     * 根据菜单ID查询信息
     *
     * @param menuId 菜单ID
     * @return 菜单信息
     */
    SysMenu selectMenuById(Long menuId);

    /**
     * 查询菜单数量
     *
     * @param parentId 菜单父ID
     * @return 结果
     */
    int selectCountMenuByParentId(Long parentId);

    /**
     * 查询菜单使用数量
     *
     * @param menuId 菜单ID
     * @return 结果
     */
    int selectCountRoleMenuByMenuId(Long menuId);

    /**
     * 新增保存菜单信息
     *
     * @param menu 菜单信息
     * @return 结果
     */
    int insertMenu(SysMenu menu);

    /**
     * 修改保存菜单信息
     *
     * @param menu 菜单信息
     * @return 结果
     */
    int updateMenu(SysMenu menu);

    /**
     * 校验菜单名称是否唯一
     *
     * @param menu 菜单信息
     * @return 结果
     */
    String checkMenuNameUnique(SysMenu menu);

    List<TreeSelect> buildMenuTreeSelect(List<SysMenu> menus);

    /**
     * 构建前端所需要树结构
     *
     * @param menus 菜单列表
     * @return 树结构列表
     */
    List<SysMenu> buildMenuTree(List<SysMenu> menus);

    /**
     * 根据角色ID查询菜单树信息
     *
     * @param roleId 角色ID
     * @return 选中菜单列表
     */
    List<Integer> selectMenuListByRoleId(Long roleId);

    /**
     * 清除该菜单类型不相关属性
     */
    void cleanUnrelatedParams(SysMenu menu);

    /**
     * 2021-11-29 Another 获取 当前用户所拥有的物业管理平台一级菜单
     * @param userId 用户id
     * @return 当前用户所拥有的物业管理平台一级菜单
     */
    List<CheckboxDto> getFirstLevelMenu(Long userId);

    /**
     * 2021-11-29 Another 获取 物业管理员已配置的模块
     * @param estateUserId 物业管理员id
     * @return 物业管理员已配置的模块
     */
    List<String> getMenuByEstateUserId(String estateUserId);

    /**
     * 2021-12-1 Another 保存 物业管理员模块配置
     * @param sysUserMenu 物业管理员模块配置
     * @return 成功/失败
     */
    boolean saveUserMenu(SysUserMenu sysUserMenu);

}
